@@ -1,13 +1,14 @@
 
 _q_exports = ["simple", "error", "publish_error", "widgets",
-              "form_demo", "dumpreq", "srcdir"]
+              "form_demo", "dumpreq", "srcdir",
+              ("favicon.ico", "q_ico")]
 
 import sys
 from quixote.demo.pages import _q_index, _q_exception_handler, dumpreq
 from quixote.demo.widgets import widgets
 from quixote.demo.integer_ui import IntegerUI
 from quixote.errors import PublishError
-from quixote.util import StaticDirectory
+from quixote.util import StaticDirectory, StaticFile
 
 def simple (request):
     # This function returns a plain text document, not HTML.
@@ -37,4 +38,4 @@ import os
 from quixote.demo import forms
 curdir = os.path.dirname(forms.__file__)
 srcdir = StaticDirectory(curdir, list_directory=1)
-
+q_ico = StaticFile(os.path.join(curdir, 'q.ico'))
