@@ -6,7 +6,7 @@ TextWidget, CheckboxWidget, etc.
 
 # created 2000/09/20, GPW
 
-__revision__ = "$Id: widget.py 20777 2003-02-21 21:21:46Z abenard $"
+__revision__ = "$Id: widget.py 24268 2004-05-18 19:47:17Z nascheme $"
 
 import struct
 from types import FloatType, IntType, ListType, StringType, TupleType
@@ -144,13 +144,14 @@ class StringWidget (Widget):
         self.maxlength = maxlength
 
 
-    def render (self, request):
+    def render (self, request, **attributes):
         return htmltag("input", xml_end=1,
                        type=self.html_type,
                        name=self.name,
                        size=self.size,
                        maxlength=self.maxlength,
-                       value=self.value)
+                       value=self.value,
+                       **attributes)
 
 
 class FileWidget (StringWidget):
@@ -192,24 +193,25 @@ class TextWidget (Widget):
       rows : int
       wrap : string
         (see an HTML book for details on text widget wrap options)
+      css_class : string
     """
 
     widget_type = "text"
 
-    def __init__ (self, name, value=None,
-                  cols=None, rows=None,
-                  wrap=None):
+    def __init__ (self, name, value=None, cols=None, rows=None, wrap=None,
+                  css_class=None):
         Widget.__init__(self, name, value)
         self.cols = cols
         self.rows = rows
         self.wrap = wrap
-
+        self.css_class = css_class
 
     def render (self, request):
         return (htmltag("textarea", name=self.name,
                         cols=self.cols,
                         rows=self.rows,
-                        wrap=self.wrap) +
+                        wrap=self.wrap,
+                        css_class=self.css_class) +
                 htmlescape(self.value or "") +
                 htmltext("</textarea>"))
 
@@ -498,12 +500,12 @@ class RadiobuttonsWidget (SingleSelectWidget):
                 checked = ValuelessAttr
             else:
                 checked = None
-            r = htmltag("input",
+            r = htmltag("input", xml_end=True,
                         type="radio",
                         name=self.name,
                         value=key,
                         checked=checked)
-            tags.append(r + htmlescape(description) + htmltext('</input>'))
+            tags.append(r + htmlescape(description))
         return htmlescape(self.delim).join(tags)
 
 
@@ -735,12 +737,19 @@ class ListWidget (Widget):
         self.args = args
 
         self.added_elements_widget = self.create_subwidget(
-            "hidden", "added_elements",
-            value="1")
+            "hidden", "added_elements")
+
+        added_elements = int(self.added_elements_widget.get_current_value() or
+                             '1')
+
         self.add_button = self.create_subwidget(
             "submit_button", "add_element",
             value="Add %s" % element_name)
 
+        if self.add_button.is_submitted():
+            added_elements += 1
+            self.added_elements_widget.set_current_value(str(added_elements))
+
         self.element_widgets = []
         self.element_count = 0
 
@@ -748,13 +757,9 @@ class ListWidget (Widget):
             for element in self.value:
                 self.add_element(element)
 
-        self.added_elements = int(
-            self.added_elements_widget.get_current_value() or 1)
-
-        for i in range(self.added_elements):
+        for index in range(added_elements):
             self.add_element()
 
-
     def add_element (self, value=None):
         self.element_widgets.append(
             self.create_subwidget(self.element_type,
@@ -763,7 +768,6 @@ class ListWidget (Widget):
                                   **self.args))
         self.element_count += 1
 
-
     def render (self, request):
         tags = []
         for element_widget in self.element_widgets:
@@ -772,12 +776,7 @@ class ListWidget (Widget):
         tags.append(self.added_elements_widget.render(request))
         return htmltext('<br />\n').join(tags)
 
-
     def parse (self, request):
-        if self.add_button.parse(request):
-            self.add_element()
-            self.added_elements += 1
-            self.added_elements_widget.value = str(self.added_elements)
         self.value = []
         for element_widget in self.element_widgets:
             value = element_widget.parse(request)
@@ -785,3 +784,62 @@ class ListWidget (Widget):
                 self.value.append(value)
         self.value = self.value or None
         return self.value
+
+
+
+class CollapsibleListWidget (ListWidget):
+    """Widget for lists of objects with associated delete buttons.
+
+    CollapsibleListWidget behaves like ListWidget except that each element
+    is rendered with an associated delete button.  Pressing the delete 
+    button will cause the associated element name to be added to a hidden
+    widget that remembers all deletions until the form is submitted.
+    Only elements that are not marked as deleted will be rendered and
+    ultimately added to the value of the widget.
+
+    Instance attributes:
+      value : [any]
+    """
+
+    widget_type = "collapsible_list"
+
+    def __init__ (self, name, value=None, element_name="row", **args):
+        self.name = name
+        self.element_name = element_name
+        self.deleted_elements_widget = self.create_subwidget(
+            "hidden", "deleted_elements")
+        self.element_delete_buttons = []
+        self.deleted_elements = (
+            self.deleted_elements_widget.get_current_value() or '')
+        ListWidget.__init__(self, name, value=value,
+                            element_name=element_name,
+                            **args)
+
+    def add_element (self, value=None):
+        element_widget_name = "element_%d" % self.element_count
+        if self.deleted_elements.find(element_widget_name) == -1:
+            delete_button = self.create_subwidget(
+                "submit_button", "delete_" + element_widget_name,
+                value="Delete %s" % self.element_name)
+            if delete_button.is_submitted():
+                self.element_count += 1
+                self.deleted_elements += element_widget_name
+                self.deleted_elements_widget.set_current_value(
+                    self.deleted_elements)
+            else:
+                self.element_delete_buttons.append(delete_button)
+                ListWidget.add_element(self, value=value)
+        else:
+            self.element_count += 1
+
+    def render (self, request):
+        tags = []
+        for element_widget, element_delete_button in zip(
+                self.element_widgets, self.element_delete_buttons):
+            if self.deleted_elements.find(element_widget.name) == -1:
+                tags.append(element_widget.render(request) +
+                            element_delete_button.render(request))
+        tags.append(self.add_button.render(request))
+        tags.append(self.added_elements_widget.render(request))
+        tags.append(self.deleted_elements_widget.render(request))
+        return htmltext('<br />\n').join(tags)
