@@ -1,6 +1,6 @@
 """quixote.http_request
-$HeadURL: svn://svn/repos/trunk/quixote/http_request.py $
-$Id: http_request.py 20965 2003-03-05 18:40:37Z akuchlin $
+$HeadURL: svn+ssh://svn/repos/trunk/quixote/http_request.py $
+$Id: http_request.py 23843 2004-04-02 15:18:05Z nascheme $
 
 Provides the HTTPRequest class and related code for parsing HTTP
 requests, such as the FileUpload class.
@@ -22,11 +22,11 @@ copyright and license from the rest of Quixote).
 #
 ##############################################################################
 
-__revision__ = "$Id: http_request.py 20965 2003-03-05 18:40:37Z akuchlin $"
+__revision__ = "$Id: http_request.py 23843 2004-04-02 15:18:05Z nascheme $"
 
 import re
 import time
-import urlparse
+import urlparse, urllib
 from cgi import FieldStorage
 from types import ListType
 
@@ -146,8 +146,7 @@ class HTTPRequest:
         """Process request inputs.
         """
         self.start_time = time.time()
-        method = self.environ.get('REQUEST_METHOD', 'GET')
-        if method != 'GET':
+        if self.get_method() != 'GET':
             # Avoid consuming the contents of stdin unless we're sure
             # there's actually form data.
             if self.content_type == "multipart/form-data":
@@ -189,6 +188,11 @@ class HTTPRequest:
     def get_form_var (self, var_name, default=None):
         return self.form.get(var_name, default)
 
+    def get_method (self):
+        """Returns the HTTP method for this request
+        """
+        return self.environ.get('REQUEST_METHOD', 'GET')
+
     def formiter (self):
         return self.form.iteritems()
 
@@ -206,9 +210,11 @@ class HTTPRequest:
         Return the server name with an optional port number, eg.
         "www.example.com" or "foo.bar.com:8000".
         """
+        http_host = self.environ.get("HTTP_HOST")
+        if http_host:
+            return http_host
         server_name = self.environ["SERVER_NAME"].strip()
         server_port = self.environ.get("SERVER_PORT")
-
         if (not server_port or
             (self.scheme == "http" and server_port == "80") or
             (self.scheme == "https" and server_port == "443")):
@@ -230,17 +236,29 @@ class HTTPRequest:
           get_path(1) == "/bar/baz"
           get_path(2) == "/bar"
 
-        Raises ValueError if n is too big for the current path.
-        """
+        If 'n' is negative, then components from the left of the path
+        are returned.  Continuing the above example,
+          get_path(-1) = "/bar"
+          get_path(-2) = "/bar/baz"
+          get_path(-3) = "/bar/baz/"
+
+        Raises ValueError if absolute value of n is larger than the number of
+        path components."""
+
         path_info = self.environ.get('PATH_INFO', '')
         path = self.environ['SCRIPT_NAME'] + path_info
         if n == 0:
             return path
         else:
             path_comps = path.split('/')
-            if n > len(path_comps)-1:
+            if abs(n) > len(path_comps)-1:
                 raise ValueError, "n=%d too big for path '%s'" % (n, path)
-            return '/'.join(path_comps[:-n])
+            if n > 0:
+                return '/'.join(path_comps[:-n])
+            elif n < 0:
+                return '/'.join(path_comps[:-n+1])
+            else:
+                assert 0, "Unexpected value for n (%s)" % n
 
     def get_url (self, n=0):
         """get_url(n : int = 0) -> string
@@ -251,7 +269,8 @@ class HTTPRequest:
         "http://foo.com/bar".  Does not include the query string (if
         any).
         """
-        return "%s://%s%s" % (self.scheme, self.get_server(), self.get_path(n))
+        return "%s://%s%s" % (self.scheme, self.get_server(),
+                              urllib.quote(self.get_path(n)))
 
     def get_environ (self, key, default=None):
         """get_environ(key : string) -> string
@@ -329,7 +348,7 @@ class HTTPRequest:
                   "<h3>environ</h3>"
                   "<table>"]
         for k,v in self.environ.items():
-            lines.append(row_fmt % (html_quote(k), html_quote(v)))
+            lines.append(row_fmt % (html_quote(k), html_quote(str(v))))
         lines.append("</table>")
 
         return "\n".join(lines)
@@ -436,7 +455,7 @@ class HTTPRequest:
             if "/" in real_ua:
                 (name, version) = real_ua.split("/", 1)
             else:
-                if real_ua.startswith("MSIE"):
+                if real_ua.startswith("MSIE") and ' ' in real_ua:
                     (name, version) = real_ua.split(" ", 1)
                 else:
                     name = real_ua
@@ -455,12 +474,12 @@ class HTTPRequest:
         """redirect(location : string, permanent : boolean = false)
            -> string
 
-        Create a redirection response.  If the location does not
-        specify the server then it is automatically added.  The return
-        value is an HTML document indicating the new URL (useful if the
-        client browser does not honor the redirect).
+        Create a redirection response.  If the location is relative, then it
+        will automatically be made absolute.  The return value is an HTML
+        document indicating the new URL (useful if the client browser does
+        not honor the redirect).
         """
-        location = urlparse.urljoin(self.get_url(),location)
+        location = urlparse.urljoin(self.get_url(), location)
         return self.response.redirect(location, permanent)
 
 
