@@ -1,25 +1,26 @@
 """quixote.publish
 $HeadURL: svn+ssh://svn/repos/trunk/quixote/publish.py $
-$Id: publish.py 21933 2003-06-26 17:29:52Z nascheme $
+$Id: publish.py 24267 2004-05-18 19:33:04Z nascheme $
 
 Logic for publishing modules and objects on the Web.
 """
 
-__revision__ = "$Id: publish.py 21933 2003-06-26 17:29:52Z nascheme $"
+__revision__ = "$Id: publish.py 24267 2004-05-18 19:33:04Z nascheme $"
 
 import sys, os, traceback, cStringIO
 import time, types, socket, re, warnings
-import cgi
 import struct
 try:
     import zlib # for COMPRESS_PAGES option
 except ImportError:
     pass
 
-from quixote import errors, fcgi
+from quixote import errors
 from quixote.html import htmltext
 from quixote.http_request import HTTPRequest, get_content_type
-from quixote.http_response import HTTPResponse
+from quixote.http_response import HTTPResponse, Stream
+from quixote.upload import HTTPUploadRequest
+from quixote.sendmail import sendmail
 
 try:
     import cgitb                        # Only available in Python 2.2
@@ -37,7 +38,8 @@ def _get_module (name):
 # plugged in here -- we can't do it now because the environment isn't
 # really setup for us yet if running as a FastCGI script.
 INTERNAL_ERROR_MESSAGE = """\
-<!DOCTYPE HTML PUBLIC "-//IETF//DTD HTML 2.0//EN">
+<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.0//EN"
+        "http://www.w3.org/TR/REC-html40/strict.dtd">
 <html>
 <head><title>Internal Server Error</title></head>
 <body>
@@ -206,7 +208,6 @@ class Publisher:
     def create_request (self, stdin, env):
         ctype = get_content_type(env)
         if ctype == "multipart/form-data":
-            from quixote.upload import HTTPUploadRequest
             req = HTTPUploadRequest(stdin, env, content_type=ctype)
             req.set_upload_dir(self.config.upload_dir,
                                self.config.upload_dir_mode)
@@ -253,7 +254,6 @@ class Publisher:
             timestamp = time.strftime('%Y-%m-%d %H:%M:%S', time.localtime(now))
 
             env = request.environ
-            method = env.get('REQUEST_METHOD')
 
             # Under Apache, REQUEST_URI == SCRIPT_NAME + PATH_INFO.
             # Not everyone uses Apache, so we have to stick to
@@ -271,7 +271,7 @@ class Publisher:
                                     str(user),
                                     timestamp,
                                     os.getpid(),
-                                    method,
+                                    request.get_method(),
                                     request_uri + query,
                                     proto,
                                     request.response.status_code,
@@ -416,16 +416,9 @@ class Publisher:
 
     def mail_error (self, msg, error_summary):
         """Send an email notifying someone of a traceback."""
-
-        cmd = "/usr/lib/sendmail '%s'" % self.config.error_email
-        pipe = os.popen(cmd, 'w')
-        pipe.write('Subject: Quixote Traceback (%s)\n' % error_summary)
-        pipe.write('From: %s <%s>\n' % (socket.gethostname(),
-                                        self.config.error_email))
-        pipe.write('To: %s\n' % self.config.error_email)
-        pipe.write('\n')
-        pipe.write(msg)
-        pipe.close()
+        sendmail('Quixote Traceback (%s)' % error_summary,
+                 msg, [self.config.error_email],
+                 from_addr=(self.config.error_email, socket.gethostname()))
 
     def get_namespace_stack (self):
         """get_namespace_stack() ->  [ module | instance | class ]
@@ -481,21 +474,22 @@ class Publisher:
 
         return output
 
-    _gzip_header = ("\037\213" # magic
+    _GZIP_HEADER = ("\037\213" # magic
                     "\010" # compression method
                     "\000" # flags
                     "\000\000\000\000" # time, who cares?
                     "\002"
                     "\377")
 
-    def compress_output (self, request, output):
+    _GZIP_THRESHOLD = 200 # responses smaller than this are not compressed
 
+    def compress_output (self, request, output):
         encoding = request.get_encoding(["gzip", "x-gzip"])
         n = len(output)
-        if n > 200 and encoding:
+        if n > self._GZIP_THRESHOLD and encoding:
             co = zlib.compressobj(6, zlib.DEFLATED, -zlib.MAX_WBITS,
                                   zlib.DEF_MEM_LEVEL, 0)
-            chunks = [self._gzip_header,
+            chunks = [self._GZIP_HEADER,
                       co.compress(output),
                       co.flush(),
                       struct.pack("<ll", zlib.crc32(output), len(output))]
@@ -505,6 +499,16 @@ class Publisher:
             request.response.set_header("Content-Encoding", encoding)
         return output
 
+    def filter_output(self, request, output):
+        """Hook for post processing the output.  Subclasses may wish to
+        override (e.g. check HTML syntax).
+        """
+        if (output and
+                self.config.compress_pages and
+                not isinstance(output, Stream)):
+            output = self.compress_output(request, str(output))
+        return output
+    
     def process_request (self, request, env):
         """process_request(request : HTTPRequest, env : dict) : string
 
@@ -513,12 +517,17 @@ class Publisher:
         exceptions will be handled here.
         """
         self._set_request(request)
-        output = self.try_publish(request, env.get('PATH_INFO', ''))
+        try:
+            self.parse_request(request)
+            output = self.try_publish(request, env.get('PATH_INFO', ''))
+        except errors.PublishError, exc:
+            # Exit the publishing loop and return a result right away.
+            output = self.finish_interrupted_request(request, exc)
+        except:
+            # Some other exception, generate error messages to the logs, etc.
+            output = self.finish_failed_request(request)
+        output = self.filter_output(request, output)
         self.log_request(request)
-
-        if output and self.config.compress_pages:
-            output = self.compress_output(request, str(output))
-
         return output
 
     def publish (self, stdin, stdout, stderr, env):
@@ -529,19 +538,11 @@ class Publisher:
         output.
         """
         request = self.create_request(stdin, env)
-        try:
-            self.parse_request(request)
-            output = self.process_request(request, env)
-        except errors.PublishError, exc:
-            # Exit the publishing loop and return a result right away.
-            output = self.finish_interrupted_request(request, exc)
-        except:
-            # Some other exception, generate error messages to the logs, etc.
-            output = self.finish_failed_request(request)
+        output = self.process_request(request, env)
 
         # Output results from Response object
         if output:
-            request.response.set_body(str(output))
+            request.response.set_body(output)
         try:
             request.response.write(stdout)
         except IOError, exc:
@@ -550,18 +551,25 @@ class Publisher:
 
 
     def publish_cgi (self):
-        """publish_cgi() -> None
+        """publish_cgi()
 
-        Entry point from CGI and FCGI scripts; it will repeatedly do the
-        publish() function until there are no more requests (only once for
-        CGI scripts).
+        Entry point from CGI scripts; it will execute the publish function
+        once and return.
         """
-
-        # On Windows sys.stdout is opened in text mode by default
         if sys.platform == "win32":
+            # On Windows sys.stdout is opened in text mode by default
             import msvcrt
             msvcrt.setmode(sys.__stdout__.fileno(), os.O_BINARY)
+        self.publish(sys.__stdin__, sys.__stdout__, sys.__stderr__, os.environ)
 
+    def publish_fcgi (self):
+        """publish_fcgi()
+
+        Entry point from FCGI scripts; it will repeatedly do the publish()
+        function until there are no more requests.  This should also work
+        for CGI scripts but it is not as portable as publish_cgi().
+        """
+        from quixote import fcgi
         while fcgi.isFCGI() and not self.exit_now:
             f = fcgi.FCGI()
             self.publish(f.inp, f.out, f.err, f.env)
@@ -569,6 +577,7 @@ class Publisher:
             if self.config.run_once:
                 break
 
+
 # class Publisher
 
 
@@ -628,11 +637,11 @@ class SessionPublisher (Publisher):
 
 _slash_pat = re.compile("//*")
 
-def _traverse_url (root_namespace, path, request, fix_trailing_slash, 
+def _traverse_url (root_namespace, path, request, fix_trailing_slash,
                    namespace_stack):
     """traverse_url(root_namespace : any, path : string,
                     request : HTTPRequest, fix_trailing_slash : bool,
-                    namespace_stack : dict) -> (object : any)
+                    namespace_stack : list) -> (object : any)
 
     Perform traversal based on the provided path, starting at the root
     object.  It returns the script name and path info values for
@@ -717,6 +726,31 @@ def _traverse_url (root_namespace, path, request, fix_trailing_slash,
     return object
 
 
+def _lookup_export (name, exports):
+    """Search an exports list for a name.  Returns the internal name for
+    'name' or return None if 'name' is not in 'exports'.
+    
+    Each element of the export list can be either a string or a 2-tuple
+    of strings that maps an external name into internal name.  The
+    mapping is useful when the desired external name is not a valid
+    Python identifier.
+    """
+    for value in exports:
+        if value == name:
+            internal_name = name
+            break
+        elif type(value) is types.TupleType:
+            if value[0] == name:
+                internal_name = value[1] # internal name is different
+                break
+    else:
+        if name == '_q_index':
+            internal_name = name # _q_index does not need to be in exports list
+        else:
+            internal_name = None # not found in exports
+    return internal_name
+
+
 def _get_component (container, component, path, request, namespace_stack):
     """Get one component of a path from a namespace.
     """
@@ -745,7 +779,22 @@ def _get_component (container, component, path, request, namespace_stack):
     # _q_lookup() doesn't exist or is None, a TraversalError is
     # raised.
 
-    if (component != "_q_index" and component not in container._q_exports):
+    # Check if component is in _q_exports.  The elements in
+    # _q_exports can be strings or 2-tuples mapping external names
+    # to internal names.
+    if component in container._q_exports or component == '_q_index':
+        internal_name = component
+    else:
+        # check for an explicit external to internal mapping
+        for value in container._q_exports:
+           if type(value) is types.TupleType:
+               if value[0] == component:
+                   internal_name = value[1]
+                   break
+        else:
+            internal_name = None
+
+    if internal_name is None:
         # Component is not in exports list.
         object = None
         if hasattr(container, "_q_lookup"):
@@ -760,32 +809,34 @@ def _get_component (container, component, path, request, namespace_stack):
                                                     container,
                                                     component))
 
-    # From here on, you can assume that the name is either in
-    # _q_exports, or the name is '_q_index'
-    elif hasattr(container, component):
+    # From here on, you can assume that the internal_name is not None
+    elif hasattr(container, internal_name):
         # attribute is in _q_exports and exists
-        object = getattr(container, component)
+        object = getattr(container, internal_name)
 
-    elif component == '_q_index':
-        raise errors.AccessError(
-            private_msg=("_q_index not found in %r" % container))
+    elif internal_name == '_q_index':
+        if hasattr(container, "_q_lookup"):
+            object = container._q_lookup(request, "")
+        else:
+            raise errors.AccessError(
+                private_msg=("_q_index not found in %r" % container))
 
     elif hasattr(container, "_q_resolve"):
-        object = container._q_resolve(component)
+        object = container._q_resolve(internal_name)
         if object is None:
-            raise RuntimeError, ("component %r listed in _q_exports, "
-                                 "but not returned by _q_resolve()"
-                                 % component)
+            raise RuntimeError, ("component listed in _q_exports, "
+                                 "but not returned by _q_resolve(%r)"
+                                 % internal_name)
         else:
             # Set the object, so _q_resolve won't need to be called again.
-            setattr(container, component, object)
+            setattr(container, internal_name, object)
 
     elif type(container) is types.ModuleType:
         # try importing it as a sub-module.  If we get an ImportError
         # here we don't catch it.  It means that something that
         # doesn't exist was exported or an exception was raised from
         # deeper in the code.
-        mod_name = container.__name__ + '.' + component
+        mod_name = container.__name__ + '.' + internal_name
         object = _get_module(mod_name)
 
     else:
