@@ -1,6 +1,6 @@
 """quixote.session
-$HeadURL: svn://svn/repos/trunk/quixote/session.py $
-$Id: session.py 20965 2003-03-05 18:40:37Z akuchlin $
+$HeadURL: svn+ssh://svn/repos/trunk/quixote/session.py $
+$Id: session.py 23946 2004-04-08 16:13:15Z nascheme $
 
 Quixote session management.  There are two levels to Quixote's
 session management system:
@@ -17,13 +17,13 @@ with putting all session data into a Session object (or, better, into an
 application-specific subclass of Session).
 
 The default implementation provided here is not persistent: when the
-Quixote process shuts down, all session data is lost.  You will need to
-subclass SessionManager if you want to implement persistent sessions.
+Quixote process shuts down, all session data is lost.  See
+doc/session-mgmt.txt for information on session persistence.
 """
 
-__revision__ = "$Id: session.py 20965 2003-03-05 18:40:37Z akuchlin $"
+__revision__ = "$Id: session.py 23946 2004-04-08 16:13:15Z nascheme $"
 
-import sys, string, binascii
+import binascii
 from time import time, localtime, strftime, clock
 
 from quixote import get_publisher
@@ -339,16 +339,27 @@ class SessionManager:
             # repeatedly storing the same object in the same mapping.
             self[session.id] = session
 
+    def _set_cookie (self, request, value, **attrs):
+        config = get_publisher().config
+        name = config.session_cookie_name
+        if config.session_cookie_path:
+            path = config.session_cookie_path
+        else:
+            path = request.environ['SCRIPT_NAME']
+            if not path.endswith("/"):
+                path += "/"
+        domain = config.session_cookie_domain
+        request.response.set_cookie(name, value, domain=domain,
+                                    path=path, **attrs)
+        return name
+        
     def set_session_cookie (self, request, session_id):
         """set_session_cookie(request : HTTPRequest, session_id : string)
 
         Ensure that a session cookie with value 'session_id' will be
         returned to the client via 'request.response'.
         """
-        config = get_publisher().config
-        request.response.set_cookie(config.session_cookie_name, session_id,
-                                    domain = config.session_cookie_domain,
-                                    path = config.session_cookie_path)
+        self._set_cookie(request, session_id)
 
     def revoke_session_cookie (self, request):
         """revoke_session_cookie(request : HTTPRequest)
@@ -358,14 +369,9 @@ class SessionManager:
         remove the cookie from 'request' so that further processing of
         this request does not see the cookie's revoked value.
         """
-        config = get_publisher().config
-        response = request.response
-        response.set_cookie(config.session_cookie_name, "",
-                            domain = config.session_cookie_domain,
-                            path = config.session_cookie_path,
-                            max_age = 0)
-        if request.cookies.has_key(config.session_cookie_name):
-            del request.cookies[config.session_cookie_name]
+        cookie_name = self._set_cookie(request, "", max_age=0)
+        if request.cookies.has_key(cookie_name):
+            del request.cookies[cookie_name]
 
     def expire_session (self, request):
         """expire_session(request : HTTPRequest)
@@ -421,13 +427,13 @@ class Session:
         an object to identify the human being on the other end of the
         line.  It's up to you whether to store just a string in 'user',
         or some more complex data structure or object.
-      __remote_address : string
+      _remote_address : string
         IP address of user owning this session (only set when the
         session is created -- requests for this session from a different
         IP address will either raise SessionError or be treated
         normally, depending on the CHECK_SESSION_ADDR config variable)
-      __creation_time : float
-      __access_time : float
+      _creation_time : float
+      _access_time : float
         two ways of keeping track of the "age" of the session.
         Note that '__access_time' is maintained by the SessionManager that
         owns this session, using _set_access_time().
@@ -437,9 +443,7 @@ class Session:
 
     Feel free to access 'id' and 'user' directly, but do not modify
     'id'.  The preferred way to set 'user' is with the set_user() method
-    (which you might want to override for type-checking).  The other
-    attributes are private -- keep your grubby hands off of them and use
-    the appropriate accessor methods.
+    (which you might want to override for type-checking).
     """
 
     MAX_FORM_TOKENS = 16 # maximum number of outstanding form tokens
@@ -447,8 +451,8 @@ class Session:
     def __init__ (self, request, id):
         self.id = id
         self.user = None
-        self.__remote_address = request.get_environ("REMOTE_ADDR")
-        self.__creation_time = self.__access_time = time()
+        self._remote_address = request.get_environ("REMOTE_ADDR")
+        self._creation_time = self._access_time = time()
         self._form_tokens = [] # queue
 
     def __repr__ (self):
@@ -485,13 +489,13 @@ class Session:
 
     def dump (self, file=None, header=1, deep=1):
         time_fmt = "%Y-%m-%d %H:%M:%S"
-        ctime = strftime(time_fmt, localtime(self.__creation_time))
-        atime = strftime(time_fmt, localtime(self.__access_time))
+        ctime = strftime(time_fmt, localtime(self._creation_time))
+        atime = strftime(time_fmt, localtime(self._access_time))
 
         if header:
             file.write('session %s:' % self.id)
         file.write('  user %s' % self.user)
-        file.write('  __remote_address: %s' % self.__remote_address)
+        file.write('  _remote_address: %s' % self._remote_address)
         file.write('  created %s, last accessed %s' % (ctime, atime))
         file.write('  _form_tokens: %s\n' % self._form_tokens)
 
@@ -530,38 +534,38 @@ class Session:
         """Return the IP address (dotted-quad string) that made the
         initial request in this session.
         """
-        return self.__remote_address
+        return self._remote_address
 
     def get_creation_time (self):
         """Return the time that this session was created (seconds
         since epoch).
         """
-        return self.__creation_time
+        return self._creation_time
 
     def get_access_time (self):
         """Return the time that this session was last accessed (seconds
         since epoch).
         """
-        return self.__access_time
+        return self._access_time
 
     def get_creation_age (self, _now=None):
         """Return the number of seconds since session was created."""
         # _now arg is not strictly necessary, but there for consistency
         # with get_access_age()
-        return (_now or time()) - self.__creation_time
+        return (_now or time()) - self._creation_time
 
     def get_access_age (self, _now=None):
         """Return the number of seconds since session was last accessed."""
         # _now arg is for SessionManager's use
-        return (_now or time()) - self.__access_time
+        return (_now or time()) - self._access_time
 
 
     # -- Methods for SessionManager only -------------------------------
 
     def _set_access_time (self, resolution):
         now = time()
-        if now - self.__access_time > resolution:
-            self.__access_time = now
+        if now - self._access_time > resolution:
+            self._access_time = now
 
 
     # -- Form token methods --------------------------------------------
